'use strict';
angular.module('aom.controllers')
.controller('LogonCtrl', function ($timeout, $scope, $state, $ionicLoading, $ionicModal, aomCustomStylesSrv, aomResourcesSrv, aomPollSrv, aomUserSrv, constants, aomCustomUriSrv, aomLogSrv, aomJobsSrv, appConfig, aomHardwareSrv) {
    aomPollSrv.stop();

    function keyboardScrolling(toggle) {
      if (ionic.Platform.isIOS() && window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard && !ionic.Platform.isWindowsPhone() && !ionic.Platform.isEdge()) {
        cordova.plugins.Keyboard.disableScroll(!toggle);
      }
    }

    keyboardScrolling(true);

    $scope.data = {
      resources: aomResourcesSrv.data.resources,
      resourcesData: aomResourcesSrv.data,
      appVersion: constants.common.appVersion,
      username: aomResourcesSrv.data.lastUser.Username,
      pwd: appConfig.defaultPassword,
      error: null,
      responsiveClass: window.screen.height > 750 ? 'responsive-lg' : 'responsive-sm',
      canMOPUserLoggedIn: aomResourcesSrv.data.lastUser.Username && aomResourcesSrv.data.MOP,
      MOP: aomResourcesSrv.data.MOP,
      OEM: aomResourcesSrv.data.OEM
    };

    $scope.onSubmit = function ($event) {
      $event.preventDefault();

      var inputs = Array.prototype.slice.call(document.querySelectorAll('input'));
      var index = (inputs.indexOf(document.activeElement) + 1);

      if (index >= inputs.length || ['BODY', 'BUTTON'].indexOf(document.activeElement.nodeName) !== -1) return $scope.logon();

      var input = inputs[index];
      input.focus();
      input.select();
    };

    $scope.logon = function () {
      $scope.data.error = null;
      if ($scope.data.username && $scope.data.pwd) {
        aomUserSrv.logonRequest.Password = $scope.data.pwd;
        aomUserSrv.logonRequest.Username = $scope.data.username;
        $ionicLoading.show()
          .then(function () {
            return aomUserSrv.logon();
          })
          .then(function () {
            keyboardScrolling(false);
            aomLogSrv.log({ message: aomUserSrv.logonRequest.Username + ' Logged on' }, aomLogSrv.levels.user, aomLogSrv.severities.info);
            aomCustomStylesSrv.initUserColours();
            aomPollSrv.start();
            aomJobsSrv.data.searchQuery = '';
            aomJobsSrv.data.setSearchQuery = '';
            $state.go(aomResourcesSrv.data.firstState);
          })
          .catch(function (err) {
            aomLogSrv.log({ message: aomUserSrv.logonRequest.Username + ' Logon failed' }, aomLogSrv.levels.user, aomLogSrv.severities.err);
            $scope.data.canMOPUserLoggedIn = false;
            var errorMessage = err.loginError ? err.loginError : err.message;
            $scope.data.error = errorMessage;
          })
          .finally(function () {
            $ionicLoading.hide();
          });
      } else {
        $scope.data.error = aomResourcesSrv.data.resources.Errors.IncompleteData;
      }
    };

    $scope.showTermsAndConditions = function () {

      $scope.hideModal = function () {
        $scope.modal.hide();
      };

      $ionicModal.fromTemplateUrl('App/Common/Templates/termsAndConditions.html', {
        scope: $scope,
        animation: aomHardwareSrv.isTomTom() ? 'none' : 'slide-in-up'
      }).then(function (modal) {
        $scope.modal = modal;
        $timeout(function () {
          $scope.modal.show();
        });
      });
    };

    aomCustomUriSrv.data.backState = 'logon';

    $scope.$on('$ionicView.afterEnter', function () {
      if ($scope.data.canMOPUserLoggedIn) {
        $timeout(function () {
          $scope.logon();
        }, 600);
      }
    });

  });