angular.module('aom.directives')
  .directive('aomRoadSelect', function (aomResourcesSrv, aomBackSrv, aomStyleSrv, aomSelectModalSrv, aomRoadsSrv) {
    return {
      scope: {
        selectObject: '=',
        changedMethod: '='
      },
      link: function ($scope) {

        var isShowingModal = false;

        $scope.data = {
          resources: aomResourcesSrv.data.resources
        };

        function setupModal() {
          return aomSelectModalSrv.setupModal($scope, 'RoadSelect');
        }

        function setData() {
          var newData = '';
          if ($scope.roads) {
            for (var i = 0; i < $scope.roads.length; i++) {
              if ($scope.roads[i].Checked) {
                newData = $scope.roads[i].Text;
                break;
              }
            }
          }
          $scope.selectObject.Data = newData;
        }

        function fireChanged() {
          if ($scope.originalValue !== $scope.selectObject.Data) {
            if ($scope.selectObject.changedMethod !== undefined) {
              $scope.selectObject.changedMethod();
            }
            if ($scope.changedMethod !== undefined) {
              $scope.changedMethod($scope.selectObject.Name);
            }
          }
        }

        function backClicked() {
          $scope.roads = [];
          isShowingModal = false;
          fireChanged();
          aomBackSrv.exit();
          aomSelectModalSrv.hideModal();
        }

        function revertOriginalValues() {
          angular.copy($scope.originalValue, $scope.selectObject.Data);
        }

        $scope.showModal = function () {
          aomBackSrv.init($scope.hideModal, 201);
          isShowingModal = true;
          $scope.originalValue = angular.copy($scope.selectObject.Data);
          $scope.data.rowClass = aomStyleSrv.getRowClass();
          setupModal()
            .then(function () {
              return $scope.modal.show();
            })
            .then(function () {
              return aomRoadsSrv.getNearest();
            })
            .then(function (roads) {
              if ($scope.selectObject.Data.length > 0) {
                var sameRoad = roads.filter(function (road) {
                  return road.Text === $scope.selectObject.Data;
                });
                if (sameRoad.length > 0) {
                  sameRoad[0].Checked = true;
                } else {
                  roads.unshift({
                    Text: $scope.selectObject.Data,
                    Checked: true
                  });
                }
              }
              $scope.roads = roads;
            });
        };

        $scope.hideModal = function (done) {
          if (done) {
            setData();
            backClicked();
          } else {
            revertOriginalValues();
            backClicked();
          }
        };

        $scope.callClick = function (index, $event) {
          if ($event !== 'change') {
            $scope.roads[index].Checked = !$scope.roads[index].Checked;
          }
          if ($event === 'change' || $event.currentTarget.id === 'parent') {
            for (var i = 0; i < $scope.roads.length; i++) {
              $scope.roads[i].Checked = i === index ? $scope.roads[i].Checked : false;
            }
          }
        };

        $scope.$on('$destroy', function () {
          if (isShowingModal) {
            setData();
          }
        });

      },
      templateUrl: 'App/Common/Directives/Select/select.html'
    };
  });