'use strict';
angular.module('aom.services')
  .factory('aomGoogleMapsSrv', function ($http, $q, $timeout, aomNetworkSrv, aomResourcesSrv, aomLocationSrv, appConfig) {
    var googleMaps = {},
      googleMapsScriptId = 'googleMaps';

    googleMaps.data = {
      initializing: false,
      apiLoaded: false,
      locationData: aomLocationSrv.data
    };

    function waitToLoad() {
      $timeout(function () {
        loadGoogleMaps();
      }, aomResourcesSrv.data.resources.MapOptions.GoogleFetchingMapInterval);
    }

    function loadGoogleMaps() {
      if (aomNetworkSrv.online()) {
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.id = googleMapsScriptId;
        script.src = `https://maps.googleapis.com/maps/api/js?v=3&key=${appConfig.googleApiKey}&libraries=places`;//browser key 1

        //script.src = 'https://maps.googleapis.com/maps/api/js?v=3&key=AIzaSyDDf3-MZav2HHIMQBHiPyBXjZrvvpZa79A';// use browser key 2 if testing http referrers

        var existingElement = document.getElementById(googleMapsScriptId);
        if (existingElement) {
          existingElement.parentNode.removeChild(existingElement);
        }
        document.body.appendChild(script);

        script.onload = function () {
          googleMaps.data.apiLoaded = true;
        };
        script.onerror = function () {
          googleMaps.data.apiLoaded = false;
          googleMaps.data.initializing = false;
          $timeout(function () {
            googleMaps.init();
          }, 5000);
        };
      } else {
        waitToLoad();
      }
    }

    googleMaps.getAddress = function (latitude, longitude) {
      var def = $q.defer();
      var geocoder = new google.maps.Geocoder();
      var latlng = new google.maps.LatLng(latitude, longitude);

      geocoder.geocode({ location: latlng }, function (results, status) {
        if (status === 'OK') {
          def.resolve(results[0].formatted_address);
        } else {
          def.reject('No Address found');
        }
      });
      return def.promise;
    };

    googleMaps.init = function () {
      if (!googleMaps.data.initializing) {
        googleMaps.data.initializing = true;
        loadGoogleMaps();
      }
    };

    return googleMaps;
  });