'use strict';
angular.module('aom.services')
  .factory('aomNetworkSrv', function ($rootScope, $cordovaNetwork, aomNotificationSrv, aomResourcesSrv) {

    var network = {};

    network.startTime = null;
    network.status = 'offline';
    network.isConnectionOnline = false;
    network.isJobServiceOnline = true;

    network.data = {
      rootLinksSet: false
    };

    function getFormattedDate() {
      var date = new Date();
      var str = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate() + " " +  date.getHours() + ":" + date.getMinutes() + ":" + date.getSeconds();
      return str;
    }

    function showOfflineToast() {
      aomNotificationSrv.notify(aomResourcesSrv.data.resources.Errors.NoInternet, false, { name: 'NetworkWatch', maxFreq: 10000 });
    }

    // Used to monitor the cordova network status or the browser online status if app is running in the browser
    function updateConnectionStatus(isOnline) {
      if (network.isConnectionOnline && !isOnline) {
        console.log('Connection is offline');
        network.isConnectionOnline = false;  
        refreshNetworkStatus();
      } else if (!network.isConnectionOnline && isOnline) {
        console.log('Connection is online');
        network.startTime = new Date();
        network.isConnectionOnline = true;
        refreshNetworkStatus();
      }
    }

    // Used to monitor the job service status
    function updateJobServiceStatus(isOnline) {
      if (network.isJobServiceOnline && !isOnline) {
        console.log('Job Service is offline');
        network.isJobServiceOnline = false;  
        refreshNetworkStatus();
      } else if (!network.isJobServiceOnline && isOnline) {
        console.log('Job Service is online');
        network.isJobServiceOnline = true;
        refreshNetworkStatus();
      }
    }

    // Update the cloud status based on the status of the connection and the job service
    function refreshNetworkStatus() {
      var newNetworkStatus = network.isConnectionOnline && network.isJobServiceOnline ? 'online' : 'offline';
      if (network.status === 'online' && newNetworkStatus === 'offline') {
        showOfflineToast();
      }
      if (network.status === 'offline' && newNetworkStatus === 'online') {
        $rootScope.$emit('$aomNetworkSrv:online');
      }
      network.status = newNetworkStatus;
      console.log('refreshNetworkStatus: ', network.status);
    }

    // All of the service functionality should purely monitor the connection status
    network.online = function () {
      return network.isConnectionOnline;
    };

    network.onlineOnWifi = function () {
      return window.cordova ? $cordovaNetwork.getNetwork() === Connection.WIFI : true;
    };

    network.onlineOnCellularNetwork = function () {
      return window.cordova ? $cordovaNetwork.getNetwork() === Connection.CELL_3G || $cordovaNetwork.getNetwork() === Connection.CELL_4G : true;
    };

    network.onlineOnEthernetForWindows = function () {
      return window.cordova ? ionic.Platform.isEdge() ? $cordovaNetwork.getNetwork() === Connection.ETHERNET : true : true;
    };

    network.onlineWithRootLinks = function () {
      return network.online() && network.data.rootLinksSet;
    };

    network.init = function () {

      console.log('Network initialisation started');

      // Window.cordova is only set when running on devices or emulators
      if (window.cordova) {
        console.log('Starting device connection state listeners');
        $rootScope.$on('$cordovaNetwork:online', function(event, networkState) {
          console.log('networkService: $cordovaNetwork:online');
          updateConnectionStatus(true);
        });
        $rootScope.$on('$cordovaNetwork:offline', function(event, networkState) {
          console.log('networkService: $cordovaNetwork:offline');
          updateConnectionStatus(false);
        });
      }
      else {
        console.log('Starting browser connection state listeners');
        window.addEventListener("online", function(e) {
          console.log('networkService: window:online');
          updateConnectionStatus(true);
        }, false);    
        window.addEventListener("offline", function(e) {
          console.log('networkService: window:offline');
          updateConnectionStatus(false);
        }, false);  
      }           

      // Monitor the status of the jobs service
      $rootScope.$on('$jobsSrv:online', function(event, jobServiceState) {
        console.log('networkService: $jobsSrv:online - ', jobServiceState, getFormattedDate());
        updateJobServiceStatus(jobServiceState);
      });      

      // Set the initial online state, this is from the original online status code and is fine for the initial check
      var isOnline = window.cordova ? navigator.connection !== undefined ? $cordovaNetwork.isOnline() : navigator.onLine : navigator.onLine
      console.log('Network initialisation completed - Status: ', isOnline);
      updateConnectionStatus(isOnline);
      
    };

    return network;

  });
